/*******************************************************************************
*
*
* File:        hal_i2c_master.h
* Description  i2c hal file
* Version      V0.1
* Date         2021-10-14
* Author       zhanghz
*******************************************************************************/

#ifndef __HAL_I2C_MASTER_H__
#define __HAL_I2C_MASTER_H__

#include "tau_device_datatype.h"
#include "tau_common.h"
#include "string.h"

/**************************************************************************
* @name : hal_i2c_m_dma_init
* @brief : i2c master dma 初始化
* @param[in] : slave_addr：目标从机地址
* @param[in] : addr_bits：目标从机地址位数
* @param[in] : i2c_speed_hz: 通信速率
* @return :
* @retval :
**************************************************************************/
void hal_i2c_m_dma_init(uint8_t slave_addr, uint8_t addr_bits, uint32_t i2c_speed_hz);

/**************************************************************************
* @name : hal_i2c_m_dma_write
* @brief : i2c master dma 发送数据
* @param[in] : txBuffer：发送数据buffer
* @param[in] : data_size：发送数据个数
* @return : STATUS_SUCCESS：数据已排入 DMA 通道，但不一定全部发送
* @return : 其它：发送出错，需要重新调用函数发送
* @retval :
**************************************************************************/
status_t hal_i2c_m_dma_write(const uint8_t *txBuffer, size_t data_size);

/**************************************************************************
* @name : hal_i2c_m_dma_read
* @brief : i2c master dma 接收数据
* @param[in] : reg_address：先发送寄存器地址给从机
* @param[in] : reg_size：地址字节数
* @param[in] : rxBuffer：接收数据buffer
* @param[in] : data_size：接收数据长度
* @return : STATUS_SUCCESS：寄存器地址发送成功，并已配置DMA接收通道，但不一定完成接收
* @return : 其它：接收出错，需要重新调用函数接收
* @retval :
**************************************************************************/
status_t hal_i2c_m_dma_read(uint32_t reg_address, size_t reg_size, uint8_t *rxBuffer, size_t data_size);

/**************************************************************************
* @name : hal_i2c_m_transfer_complate
* @brief : 获取 i2c master 发送状态
* @param[in] :
* @return : true：数据发送完成
* @return : false：数据还在发送
* @retval :
**************************************************************************/
bool hal_i2c_m_transfer_complate(void);

/**************************************************************************
* @name : hal_i2c_m_set_high_impedance
* @brief : 将 I2C 主机的IO口设置为高阻态
* @param[in] :
* @return :
* @retval :
**************************************************************************/
void hal_i2c_m_set_high_impedance(void);
/**************************************************************************
* @name : hal_i2c_m_deinit
* @brief : i2c主机 IP去初始化（关掉使能、外设时钟）
* @param[in] :
* @return :
* @retval :
***************************************************************************/
void hal_i2c_m_deinit(void);
#endif /* __HAL_I2C_MASTER_H__*/

