/*******************************************************************************
*
*
* File:        hal_spi_touch.h
* Description  spi hal file
* Version      V0.1
* Date         2021-10-25
* Author       zhanghz
*******************************************************************************/

#ifndef __HAL_SPI_MASTER_H__
#define __HAL_SPI_MASTER_H__

#include "tau_device_datatype.h"
#include "tau_common.h"
#include "string.h"

/**************************************************************************
* @name : hal_spi_m_dma_init
* @brief : SPIM DMA 初始化
* @param[in] :speed：配置通信速率
* @param[in] :cpha: 配置第一个时钟沿或者第二个时钟沿有效
* @param[in] :cpol: 配置总线空闲时时钟电平
* @return :
* @retval :
**************************************************************************/
void hal_spi_m_dma_init(uint32_t speed, uint8_t cpha, uint8_t cpol);

/**************************************************************************
* @name : hal_spi_m_dma_write
* @brief : 用SPIM 发送数据
* @param[in] :data_buffer: 发送数据 buffer 头地址
* @param[in] :data_size: 发送数据 buffer 长度
* @return :STATUS_SUCCESS: 配置成功，但数据不一定发送完成
* @return :其它：配置不成功，需要重新配置发送
* @retval :
**************************************************************************/
status_t hal_spi_m_dma_write(const uint8_t *data_buffer, size_t data_size);

/**************************************************************************
* @name : hal_spi_m_dma_read
* @brief : 用SPIM 读取数据
* @param[in] :cmd: 发送命令 buffer 头地址
* @param[in] :cmd_size: 发送命令 buffer 长度
* @param[in] :data_buffer: 读取数据 buffer 头地址
* @param[in] :data_size: 发送命令 和 读取数据 buffer 长度
* @return :STATUS_SUCCESS: 配置成功，但数据不一定读取完成
* @return :其它：配置不成功，需要重新配置发送
* @retval :
**************************************************************************/
status_t hal_spi_m_dma_read(const uint8_t *cmd, size_t cmd_size, uint8_t *data_buffer, size_t data_size);

/**************************************************************************
* @name : hal_spi_m_get_transfer_complate
* @brief : 获取 SPIM 通信完成状态
* @param[in] :
* @return :true：通信完成
* @retval :
**************************************************************************/
bool hal_spi_m_get_transfer_complate(void);

/**************************************************************************
* @name : hal_spi_m_clear_rxfifo
* @brief : 清空 rxfifo 中的数据
* @param[in] :
* @return :
* @retval :
**************************************************************************/
void hal_spi_m_clear_rxfifo(void);

/**************************************************************************
* @name : hal_spi_m_set_high_impedance
* @brief : 将 SPI 主机的IO口设置为高阻态
* @param[in] :
* @return :
* @retval :
**************************************************************************/
void hal_spi_m_set_high_impedance(void);
/**************************************************************************
* @name : hal_spi_m_deinit
* @brief : 将 SPI 主机去初始化(关掉SPIM)
* @param[in] :
* @return :true
* @retval :
**************************************************************************/
bool hal_spi_m_deinit(void);

#endif

