/*******************************************************************************
*
*
* File:          hal_swire.h
* Description： swire HAL层头文件
* Version：      V0.1
* Date：         2021-03-17
* Author：       wuc
 *******************************************************************************/
#ifndef __HAL_SWIRE_H__
#define __HAL_SWIRE_H__

/*******************************************************************************
* 1.Included files
*******************************************************************************/
#include "tau_device_datatype.h"
#include "tau_common.h"

/*******************************************************************************
* 2.Global constant and macro definitions using #define
*******************************************************************************/

/*******************************************************************************
* 3.Global structures, unions and enumerations using typedef
*******************************************************************************/

/*******************************************************************************
* 4.Global variable extern declarations
*******************************************************************************/

/*******************************************************************************
* 5.Global function prototypes
*******************************************************************************/
/**
* @brief  SWIRE初始化
* @param  无
* @retval 无
*/
void hal_swire_init(void);

/**
* @brief  SWIRE反初始化
* @param  无
* @retval 无
*/
void hal_swire_deinit(void);

/**
* @brief  配置SWIRE脉冲并开始输出
* @param  start_time：起始时长，单位us
* @param  stop_time：结束时长，单位us，必须大于300us
* @param  high_time：高电平时长，单位us
* @param  low_time：低电平时长，单位us
* @param  pulse：上升沿个数
* @retval 无
*/
void hal_swire_start(uint32_t start_time, uint32_t stop_time,
                     uint32_t high_time, uint32_t low_time,
                     uint32_t pulse);

/**
* @brief  打开或关闭背光
* @param  state：开关控制
* @retval 无
*/
void hal_swire_open(function_state_e state);

/**
* @brief  注册回调函数
* @param  cb_func：回调函数地址
* @retval 无
*/
void hal_swire_register_callback(fcb_type cb_func);

#endif /* __HAL_SWIRE_H__ */
